import { loadSchema } from '@graphql-tools/load';
import { fileURLToPath } from 'url';
import { relative } from 'pathe';
import { defu } from 'defu';
import { useLogger, resolveFiles, defineNuxtModule, resolveAlias, createResolver, addImports, addServerImports, addTemplate, addServerHandler, addPlugin } from '@nuxt/kit';
import { onDevToolsInitialized, extendServerRpc } from '@nuxt/devtools-kit';
import { existsSync } from 'fs';
import { GraphqlMiddlewareTemplate } from '../dist/runtime/settings/index.js';
import { promises, existsSync as existsSync$1 } from 'node:fs';
import { generate } from '@graphql-codegen/cli';
import * as PluginSchemaAst from '@graphql-codegen/schema-ast';
import { basename } from 'node:path';
import { Source, parse, printSourceLocation } from 'graphql';
import { Generator } from 'graphql-typescript-deluxe';
import { pascalCase } from 'change-case-all';
import colors from 'picocolors';
import { validateGraphQlDocuments } from '@graphql-tools/utils';

const name = "nuxt-graphql-middleware";
const version = "5.0.0-alpha.2";

const DEVTOOLS_UI_ROUTE = "/__nuxt-graphql-middleware";
const DEVTOOLS_UI_LOCAL_PORT = 3300;
function setupDevToolsUI(nuxt, clientPath) {
  const isProductionBuild = existsSync(clientPath);
  if (isProductionBuild) {
    nuxt.hook("vite:serverCreated", async (server) => {
      const sirv = await import('sirv').then((r) => r.default || r);
      server.middlewares.use(
        DEVTOOLS_UI_ROUTE,
        sirv(clientPath, { dev: true, single: true })
      );
    });
  } else {
    nuxt.hook("vite:extendConfig", (config) => {
      config.server = config.server || {};
      config.server.proxy = config.server.proxy || {};
      config.server.proxy[DEVTOOLS_UI_ROUTE] = {
        target: "http://localhost:" + DEVTOOLS_UI_LOCAL_PORT + DEVTOOLS_UI_ROUTE,
        changeOrigin: true,
        followRedirects: true,
        rewrite: (path) => path.replace(DEVTOOLS_UI_ROUTE, "")
      };
    });
  }
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-graphql-middleware",
      // title to display in the tab
      title: "GraphQL Middleware",
      // any icon from Iconify, or a URL to an image
      icon: "akar-icons:graphql-fill",
      // iframe view
      view: {
        type: "iframe",
        src: DEVTOOLS_UI_ROUTE
      }
    });
  });
}

function pluginLoader(name) {
  switch (name) {
    case "@graphql-codegen/schema-ast":
      return Promise.resolve(PluginSchemaAst);
  }
  throw new Error(`graphql-codegen plugin not found: ${name}`);
}
function generateSchema(moduleOptions, dest, writeToDisk) {
  const pluginConfig = moduleOptions.codegenSchemaConfig?.urlSchemaOptions;
  const schemaAstConfig = moduleOptions.codegenSchemaConfig?.schemaAstConfig || {
    sort: true
  };
  const config = {
    schema: moduleOptions.graphqlEndpoint,
    pluginLoader,
    silent: true,
    errorsOnly: true,
    config: pluginConfig,
    generates: {
      [dest]: {
        plugins: ["schema-ast"],
        config: schemaAstConfig
      }
    }
  };
  return generate(config, writeToDisk).then((v) => v[0]);
}

const logger = useLogger(name);
const defaultOptions = {
  downloadSchema: true,
  schemaPath: "~~/schema.graphql",
  serverApiPrefix: "/api/graphql_middleware",
  graphqlEndpoint: "",
  debug: false,
  includeComposables: true,
  documents: [],
  devtools: true
};
function validateOptions(options) {
  if (!options.graphqlEndpoint) {
    throw new Error("Missing graphqlEndpoint.");
  }
}
async function getSchemaPath(schemaPath, options, resolver, writeToDisk = false) {
  const dest = resolver(schemaPath);
  if (!options.downloadSchema) {
    const fileExists2 = await promises.access(dest).then(() => true).catch(() => false);
    if (!fileExists2) {
      logger.error(
        '"downloadSchema" is set to false but no schema exists at ' + dest
      );
      throw new Error("Missing GraphQL schema.");
    }
    const schemaContent = await promises.readFile(dest).then((v) => v.toString());
    return { schemaPath, schemaContent };
  }
  if (!options.graphqlEndpoint) {
    throw new Error("Missing graphqlEndpoint config.");
  }
  const result = await generateSchema(options, dest, writeToDisk);
  return { schemaPath, schemaContent: result.content };
}
const fileExists = (path, extensions = ["js", "ts", "mjs"]) => {
  if (!path) {
    return null;
  } else if (existsSync$1(path)) {
    return path;
  }
  const extension = extensions.find(
    (extension2) => existsSync$1(`${path}.${extension2}`)
  );
  return extension ? `${path}.${extension}` : null;
};

function groupOperationsByType(ops) {
  const result = {
    query: {},
    mutation: {},
    subscription: {}
  };
  for (const op of ops) {
    result[op.operationType][op.graphqlName] = {
      hasVariables: op.hasVariables,
      variablesOptional: !op.needsVariables
    };
  }
  return result;
}
function buildOperationTypeCode(operationMetadata, typeName, serverApiPrefix) {
  const imports = [];
  const resultTypes = [];
  let code = "";
  let nitroCode = "";
  const operationNames = Object.keys(operationMetadata);
  if (operationNames.length === 0) {
    return { code, nitroCode, imports, resultTypes };
  }
  const lines = [];
  const nitroLines = [];
  for (const name of operationNames) {
    const nameResult = pascalCase(`${name}${typeName}`);
    const nameVariables = pascalCase(`${name}${typeName}Variables`);
    resultTypes.push(nameResult);
    imports.push(nameResult);
    const { hasVariables, variablesOptional } = operationMetadata[name];
    if (hasVariables) {
      imports.push(nameVariables);
    }
    const variablesType = hasVariables ? nameVariables : "null";
    lines.push(
      `    ${name}: [${variablesType}, ${variablesOptional ? "true" : "false"}, ${nameResult}]`
    );
    nitroLines.push(`
    '${serverApiPrefix}/${typeName.toLowerCase()}/${name}': {
      'default': GraphqlResponse<${nameResult}>
    }`);
  }
  code += `  export type GraphqlMiddleware${typeName} = {
${lines.join(",\n")}
  }
`;
  nitroCode += nitroLines.join("\n");
  return { code, nitroCode, imports, resultTypes };
}
function generateContextTemplate(collectedOperations, serverApiPrefix) {
  const grouped = groupOperationsByType(collectedOperations);
  const queryResult = buildOperationTypeCode(
    grouped.query,
    "Query",
    serverApiPrefix
  );
  const mutationResult = buildOperationTypeCode(
    grouped.mutation,
    "Mutation",
    serverApiPrefix
  );
  const subscriptionResult = buildOperationTypeCode(
    grouped.subscription,
    "Subscription",
    serverApiPrefix
  );
  const allImports = [
    ...queryResult.imports,
    ...mutationResult.imports,
    ...subscriptionResult.imports
  ];
  const allResultTypes = [
    ...queryResult.resultTypes,
    ...mutationResult.resultTypes,
    ...subscriptionResult.resultTypes
  ];
  const combinedCode = [
    queryResult.code,
    mutationResult.code,
    subscriptionResult.code
  ].join("\n");
  const combinedNitroCode = [
    queryResult.nitroCode,
    mutationResult.nitroCode,
    subscriptionResult.nitroCode
  ].join("\n");
  return `
import type { GraphqlResponse } from '#graphql-middleware-server-options-build'
import type {
  ${allImports.join(",\n  ")}
} from './../graphql-operations'

declare module '#nuxt-graphql-middleware/generated-types' {
  export type GraphqlMiddlewareResponseUnion = ${allResultTypes.join(" | ")}
${combinedCode}
}

declare module 'nitropack' {
  interface InternalApi {
${combinedNitroCode}
  }
}
`;
}

function getMaxLengths(entries) {
  let name = 0;
  let path = 0;
  let type = 0;
  for (const entry of entries) {
    if (entry.type.length > type) {
      type = entry.type.length;
    }
    if (entry.name.length > name) {
      name = entry.name.length;
    }
    if (entry.path.length > path) {
      path = entry.path.length;
    }
  }
  return { name, path, type };
}
function logAllEntries(entries) {
  const lengths = getMaxLengths(entries);
  let prevHadError = false;
  for (const entry of entries) {
    const hasErrors = entry.errors.length > 0;
    const icon = hasErrors ? colors.red("x") : colors.green("\u2714");
    const type = entry.type.padEnd(lengths.type);
    const namePadded = colors.bold(entry.name.padEnd(lengths.name));
    const name = hasErrors ? colors.red(namePadded) : colors.green(namePadded);
    const path = colors.dim(entry.path);
    const parts = [icon, type, name, path];
    if (hasErrors && !prevHadError) {
      process.stdout.write("-".repeat(process.stdout.columns) + "\n");
    }
    logger.log(parts.join(" | "));
    if (hasErrors) {
      const errorLines = [];
      entry.errors.forEach((error) => {
        let output = colors.red(error.message);
        if (error.source && error.locations) {
          for (const location of error.locations) {
            output += "\n\n" + colors.red(printSourceLocation(error.source, location));
          }
        }
        errorLines.push(output);
      });
      logger.log(
        errorLines.join("\n").split("\n").map((v) => "    " + v).join("\n")
      );
      process.stdout.write("-".repeat(process.stdout.columns) + "\n");
    }
    prevHadError = hasErrors;
  }
  logger.restoreStd();
}
class CollectedFile {
  filePath;
  fileContents;
  isOnDisk;
  parsed;
  constructor(filePath, fileContents, isOnDisk = false) {
    this.filePath = filePath;
    this.fileContents = fileContents;
    this.isOnDisk = isOnDisk;
    this.parsed = parse(fileContents);
  }
  static async fromFilePath(filePath) {
    const content = (await promises.readFile(filePath)).toString();
    return new CollectedFile(filePath, content, true);
  }
  /**
   * If isOnDisk, re-read file contents from disk, then parse it (syntax only).
   */
  async update() {
    if (this.isOnDisk) {
      const newContents = (await promises.readFile(this.filePath)).toString();
      if (newContents === this.fileContents) {
        return false;
      }
      this.fileContents = newContents;
      this.parsed = parse(newContents);
      return true;
    }
    return false;
  }
}
class Collector {
  constructor(schema, context, nuxtConfigDocuments = [], generatorOptions) {
    this.schema = schema;
    this.context = context;
    this.nuxtConfigDocuments = nuxtConfigDocuments;
    this.generator = new Generator(schema, generatorOptions);
  }
  /**
   * All collected files.
   */
  files = /* @__PURE__ */ new Map();
  /**
   * The code generator.
   */
  generator;
  /**
   * A map of operation name and timestamp when the operation was last validated.
   */
  operationTimestamps = /* @__PURE__ */ new Map();
  /**
   * The generated operations and fragments.
   */
  rpcItems = /* @__PURE__ */ new Map();
  /**
   * The generated TypeScript type template output.
   */
  outputTypes = "";
  /**
   * The generated oeprations file.
   */
  outputOperations = "";
  /**
   * The generated context template file.
   */
  outputContext = "";
  /**
   * Whether we need to rebuild the Generator state.
   */
  needsRebuild = false;
  filePathToRelative(filePath) {
    return "./" + relative(this.context.buildDir, filePath);
  }
  operationToLogEntry(operation, errors) {
    return {
      name: operation.graphqlName,
      type: operation.operationType,
      path: operation.filePath,
      errors
    };
  }
  /**
   * Executes code gen and performs validation for operations.
   */
  buildState() {
    const output = this.generator.build();
    const operations = output.getCollectedOperations();
    const generatedCode = output.getGeneratedCode();
    this.outputOperations = output.getOperationsFile();
    this.outputTypes = output.getEverything();
    this.outputContext = generateContextTemplate(
      operations,
      this.context.serverApiPrefix
    );
    const fragmentMap = /* @__PURE__ */ new Map();
    const operationSourceMap = /* @__PURE__ */ new Map();
    for (const code of generatedCode) {
      if (code.type === "fragment" && code.graphqlName && code.source) {
        fragmentMap.set(code.graphqlName, code.source);
      } else if (code.type === "operation" && code.graphqlName && code.source) {
        operationSourceMap.set(code.graphqlName, code.source);
      }
    }
    let hasErrors = false;
    const logEntries = [];
    for (const operation of operations) {
      const previousTimestamp = this.operationTimestamps.get(
        operation.graphqlName
      );
      if (previousTimestamp === operation.timestamp) {
        continue;
      }
      const fragments = operation.getGraphQLFragmentDependencies().map((v) => fragmentMap.get(v) || "").join("\n");
      const fullOperation = operationSourceMap.get(operation.graphqlName) + fragments;
      const source = new Source(fullOperation, basename(operation.filePath));
      const document = parse(source);
      const errors = validateGraphQlDocuments(this.schema, [document]);
      if (errors.length) {
        hasErrors = true;
      } else {
        this.operationTimestamps.set(operation.graphqlName, operation.timestamp);
      }
      logEntries.push(this.operationToLogEntry(operation, errors));
    }
    logAllEntries(logEntries);
    if (hasErrors) {
      throw new Error("GraphQL errors");
    }
    for (const code of generatedCode) {
      const id = `${code.identifier}_${code.graphqlName}`;
      if (code.identifier === "fragment" || code.identifier === "mutation" || code.identifier === "query") {
        if (this.rpcItems.get(id)?.timestamp === code.timestamp) {
          continue;
        }
        const fragmentDepdendencies = code.getGraphQLFragmentDependencies().map((name) => fragmentMap.get(name) || "").join("\n\n");
        this.rpcItems.set(id, {
          id,
          timestamp: code.timestamp,
          source: code.source + "\n\n" + fragmentDepdendencies,
          name: code.graphqlName,
          filePath: code.filePath,
          identifier: code.identifier
        });
      }
    }
  }
  /**
   * Get all file paths that match the import patterns.
   */
  async getImportPatternFiles() {
    if (this.context.patterns.length) {
      return resolveFiles(this.context.srcDir, this.context.patterns, {
        followSymbolicLinks: false
      });
    }
    return [];
  }
  /**
   * Initialise the collector.
   */
  async init() {
    const files = await this.getImportPatternFiles();
    for (const filePath of files) {
      await this.addFile(filePath);
    }
    this.nuxtConfigDocuments.forEach((docString, i) => {
      const pseudoPath = `nuxt.config.ts[${i}]`;
      const file = new CollectedFile(pseudoPath, docString, false);
      this.files.set(pseudoPath, file);
      this.generator.add({
        filePath: this.filePathToRelative("nuxt.config.ts"),
        documentNode: file.parsed
      });
    });
    this.buildState();
    logger.success("All GraphQL documents are valid.");
  }
  /**
   * Add a file.
   */
  async addFile(filePath) {
    const file = await CollectedFile.fromFilePath(filePath);
    this.files.set(filePath, file);
    this.generator.add({
      filePath: this.filePathToRelative(filePath),
      documentNode: file.parsed
    });
    return file;
  }
  async handleAdd(filePath) {
    await this.addFile(filePath);
    return true;
  }
  async handleChange(filePath) {
    const file = this.files.get(filePath);
    if (!file) {
      return false;
    }
    const needsUpdate = await file.update();
    if (!needsUpdate) {
      return false;
    }
    this.generator.update({
      filePath: this.filePathToRelative(filePath),
      documentNode: file.parsed
    });
    return true;
  }
  handleUnlink(filePath) {
    const file = this.files.get(filePath);
    if (!file) {
      return false;
    }
    this.files.delete(filePath);
    this.generator.remove(this.filePathToRelative(filePath));
    return true;
  }
  handleUnlinkDir(folderPath) {
    let anyHasChanged = false;
    for (const filePath of [...this.files.keys()]) {
      if (filePath.startsWith(folderPath)) {
        const hasChanged = this.handleUnlink(filePath);
        if (hasChanged) {
          anyHasChanged = true;
        }
      }
    }
    return anyHasChanged;
  }
  /**
   * Handle the watcher event for the given file path.
   */
  async handleWatchEvent(event, filePath) {
    let hasChanged = false;
    try {
      if (event === "add") {
        hasChanged = await this.handleAdd(filePath);
      } else if (event === "change") {
        hasChanged = await this.handleChange(filePath);
      } else if (event === "unlink") {
        hasChanged = this.handleUnlink(filePath);
      } else if (event === "unlinkDir") {
        hasChanged = this.handleUnlinkDir(filePath);
      } else if (event === "addDir") {
      }
      if (hasChanged) {
        this.buildState();
      }
    } catch (e) {
      this.generator.resetCaches();
      logger.error("Failed to update GraphQL code.");
      logger.error(e);
      return false;
    }
    if (hasChanged) {
      logger.success("Finished GraphQL code update.");
    }
    return hasChanged;
  }
  /**
   * Get the TypeScript types template contents.
   */
  getTemplateTypes() {
    return this.outputTypes;
  }
  /**
   * Get the context template contents.
   */
  getTemplateContext() {
    return this.outputContext;
  }
  /**
   * Get the operations template contents.
   */
  getTemplateOperations() {
    return this.outputOperations;
  }
}

const RPC_NAMESPACE = "nuxt-graphql-middleware";
const module = defineNuxtModule({
  meta: {
    name,
    configKey: "graphqlMiddleware",
    version,
    compatibility: {
      nuxt: ">=3.13.0"
    }
  },
  defaults: defaultOptions,
  async setup(passedOptions, nuxt) {
    const options = defu({}, passedOptions, defaultOptions);
    function addAlias(name2, aliasPath) {
      nuxt.options.alias[name2] = aliasPath;
    }
    const isModuleBuild = process.env.MODULE_BUILD === "true" && nuxt.options._prepare;
    if (isModuleBuild) {
      options.graphqlEndpoint = "http://localhost";
      options.downloadSchema = false;
      options.schemaPath = "~~/schema.graphql";
      options.autoImportPatterns = [
        "~~/playground/**/*.{gql,graphql}",
        "!node_modules"
      ];
    }
    if (!passedOptions.autoImportPatterns) {
      options.autoImportPatterns = ["~~/**/*.{gql,graphql}", "!node_modules"];
    }
    options.autoImportPatterns = (options.autoImportPatterns || []).map(
      (pattern) => {
        return resolveAlias(pattern);
      }
    );
    if (!nuxt.options._prepare) {
      validateOptions(options);
    }
    const moduleResolver = createResolver(import.meta.url);
    const serverResolver = createResolver(nuxt.options.serverDir);
    const srcResolver = createResolver(nuxt.options.srcDir);
    const appResolver = createResolver(nuxt.options.dir.app);
    const rootDir = nuxt.options.rootDir;
    const rootResolver = createResolver(rootDir);
    const { schemaPath, schemaContent } = await getSchemaPath(
      resolveAlias(options.schemaPath),
      options,
      rootResolver.resolve,
      options.downloadSchema
    );
    const schema = await loadSchema(schemaContent, {
      loaders: []
    });
    const runtimeDir = fileURLToPath(new URL("./runtime", import.meta.url));
    nuxt.options.build.transpile.push(runtimeDir);
    const context = {
      patterns: options.autoImportPatterns || [],
      srcDir: nuxt.options.srcDir,
      buildDir: srcResolver.resolve(nuxt.options.buildDir),
      schemaPath,
      serverApiPrefix: options.serverApiPrefix
    };
    const collector = new Collector(
      schema,
      context,
      options.documents,
      options.codegenConfig
    );
    await collector.init();
    const isDevToolsEnabled = nuxt.options.dev && options.devtools;
    let rpc;
    if (isDevToolsEnabled) {
      const clientPath = moduleResolver.resolve("./client");
      setupDevToolsUI(nuxt, clientPath);
      onDevToolsInitialized(() => {
        rpc = extendServerRpc(RPC_NAMESPACE, {
          // register server RPC functions
          getModuleOptions() {
            return options;
          },
          getDocuments() {
            return [...collector.rpcItems.values()];
          }
        });
      });
    }
    nuxt.options.runtimeConfig.public["nuxt-graphql-middleware"] = {
      serverApiPrefix: options.serverApiPrefix
    };
    nuxt.options.appConfig.graphqlMiddleware = {
      clientCacheEnabled: !!options.clientCache?.enabled,
      clientCacheMaxSize: options.clientCache?.maxSize || 100
    };
    nuxt.options.runtimeConfig.graphqlMiddleware = {
      graphqlEndpoint: options.graphqlEndpoint || ""
    };
    if (options.includeComposables) {
      const nuxtComposables = [
        "useGraphqlQuery",
        "useGraphqlMutation",
        "useGraphqlState",
        "useAsyncGraphqlQuery"
      ];
      if (options.enableFileUploads) {
        nuxtComposables.push("useGraphqlUploadMutation");
      }
      nuxtComposables.forEach((name2) => {
        addImports({
          from: moduleResolver.resolve("./runtime/composables/" + name2),
          name: name2
        });
      });
      const serverUtils = ["useGraphqlQuery", "useGraphqlMutation"].map(
        (name2) => {
          return {
            from: moduleResolver.resolve("./runtime/server/utils/" + name2),
            name: name2
          };
        }
      );
      addServerImports(serverUtils);
    }
    const templateTypescript = addTemplate({
      filename: GraphqlMiddlewareTemplate.OperationTypes,
      write: true,
      getContents: () => collector.getTemplateTypes()
    });
    addAlias("#graphql-operations", templateTypescript.dst);
    const templateDocuments = addTemplate({
      filename: GraphqlMiddlewareTemplate.Documents,
      write: true,
      getContents: () => collector.getTemplateOperations()
    });
    addAlias("#graphql-documents", templateDocuments.dst);
    const templateContext = addTemplate({
      filename: GraphqlMiddlewareTemplate.ComposableContext,
      write: true,
      getContents: () => collector.getTemplateContext()
    });
    addAlias("#nuxt-graphql-middleware/generated-types", templateContext.dst);
    addTemplate({
      write: true,
      filename: "nuxt-graphql-middleware/graphql-documents.d.ts",
      getContents: () => {
        return `
import type {
  GraphqlMiddlewareQuery,
  GraphqlMiddlewareMutation,
} from '#nuxt-graphql-middleware/generated-types'

declare module '#graphql-documents' {
  type Operations = {
    query: GraphqlMiddlewareQuery
    mutation: GraphqlMiddlewareMutation
  }
  const operations: Operations
  export { operations, Operations }
}
`;
      }
    });
    const findServerOptions = () => {
      const newPath = serverResolver.resolve("graphqlMiddleware.serverOptions");
      const serverPath = fileExists(newPath);
      if (serverPath) {
        return serverPath;
      }
      const candidates = [
        rootResolver.resolve("graphqlMiddleware.serverOptions"),
        rootResolver.resolve("app/graphqlMiddleware.serverOptions"),
        srcResolver.resolve("graphqlMiddleware.serverOptions")
      ];
      for (let i = 0; i < candidates.length; i++) {
        const path = candidates[i];
        const filePath = fileExists(path);
        if (filePath) {
          logger.warn(
            `The graphqlMiddleware.serverOptions file should be placed in Nuxt's <serverDir> ("${nuxt.options.serverDir}/graphqlMiddleware.serverOptions.ts"). The new path will be enforced in the next major release.`
          );
          return filePath;
        }
      }
      logger.info("No graphqlMiddleware.serverOptions file found.");
    };
    const resolvedPath = findServerOptions();
    const moduleTypesPath = relative(
      nuxt.options.buildDir,
      moduleResolver.resolve("./types")
    );
    const resolvedPathRelative = resolvedPath ? relative(nuxt.options.buildDir, resolvedPath) : null;
    const template = addTemplate({
      filename: "graphqlMiddleware.serverOptions.mjs",
      write: true,
      getContents: () => {
        const serverOptionsLine = resolvedPathRelative ? `import serverOptions from '${resolvedPathRelative}'` : `const serverOptions = {}`;
        return `
${serverOptionsLine}
export { serverOptions }
`;
      }
    });
    addTemplate({
      filename: "graphqlMiddleware.serverOptions.d.ts",
      write: true,
      getContents: () => {
        const serverOptionsLineTypes = resolvedPathRelative ? `import serverOptions from '${resolvedPathRelative}'` : `const serverOptions: GraphqlMiddlewareServerOptions = {}`;
        return `
import type { GraphqlMiddlewareServerOptions } from '${moduleTypesPath}'
${serverOptionsLineTypes}
import type { GraphqlServerResponse } from '${runtimeTypesPath}'
import type { GraphqlMiddlewareResponseUnion } from '#nuxt-graphql-middleware/generated-types'

type GraphqlResponseAdditions =
  typeof serverOptions extends GraphqlMiddlewareServerOptions<infer R, any, any> ? R : {}

export type GraphqlResponse<T> = GraphqlServerResponse<T> & GraphqlResponseAdditions

export type GraphqlResponseTyped = GraphqlResponse<GraphqlMiddlewareResponseUnion>

export { serverOptions }
`;
      }
    });
    const getClientOptionsImport = () => {
      const clientOptionsPath = appResolver.resolve(
        "graphqlMiddleware.clientOptions"
      );
      if (fileExists(clientOptionsPath)) {
        const pathRelative = relative(nuxt.options.buildDir, clientOptionsPath);
        return `import clientOptions from '${pathRelative}'`;
      }
    };
    const clientOptionsImport = getClientOptionsImport();
    const clientOptionsTemplate = addTemplate({
      filename: "graphqlMiddleware.clientOptions.mjs",
      write: true,
      getContents: () => {
        if (clientOptionsImport) {
          return `${clientOptionsImport}
export { clientOptions }`;
        }
        return `export const clientOptions = {}`;
      }
    });
    const runtimeTypesPath = relative(
      nuxt.options.buildDir,
      moduleResolver.resolve("./runtime/types.ts")
    );
    addTemplate({
      filename: "graphqlMiddleware.clientOptions.d.ts",
      write: true,
      getContents: () => {
        if (clientOptionsImport) {
          return `import type { GraphqlClientOptions } from '${runtimeTypesPath}'
${clientOptionsImport}

export type GraphqlClientContext = typeof clientOptions extends GraphqlClientOptions<infer R> ? R : {}

export { clientOptions }`;
        }
        return `import type { GraphqlClientOptions } from '${runtimeTypesPath}'
export const clientOptions: GraphqlClientOptions

export type GraphqlClientContext = {}
`;
      }
    });
    addAlias("#graphql-middleware-client-options", clientOptionsTemplate.dst);
    nuxt.options.nitro.externals = nuxt.options.nitro.externals || {};
    nuxt.options.nitro.externals.inline = nuxt.options.nitro.externals.inline || [];
    nuxt.options.nitro.externals.inline.push(template.dst);
    nuxt.options.nitro.externals.inline.push(templateDocuments.dst);
    addAlias("#graphql-middleware-server-options-build", template.dst);
    addAlias(
      "#graphql-middleware/types",
      moduleResolver.resolve("./runtime/types.ts")
    );
    addServerHandler({
      handler: moduleResolver.resolve("./runtime/serverHandler/index"),
      route: options.serverApiPrefix + "/:operation/:name"
    });
    if (options.enableFileUploads) {
      addServerHandler({
        handler: moduleResolver.resolve("./runtime/serverHandler/upload"),
        route: options.serverApiPrefix + "/upload/:name"
      });
    }
    addPlugin(moduleResolver.resolve("./runtime/plugins/provideState"), {
      append: false
    });
    nuxt.hook("nitro:config", (nitroConfig) => {
      nitroConfig.externals = defu(
        typeof nitroConfig.externals === "object" ? nitroConfig.externals : {},
        {
          inline: [moduleResolver.resolve("./runtime")]
        }
      );
    });
    if (nuxt.options.dev || nuxt.options._prepare) {
      addServerHandler({
        handler: moduleResolver.resolve("./runtime/serverHandler/debug"),
        route: options.serverApiPrefix + "/debug"
      });
      nuxt.hook("builder:watch", async (event, pathAbsolute) => {
        if (pathAbsolute === schemaPath) {
          return;
        }
        if (!pathAbsolute.match(/\.(gql|graphql)$/)) {
          return;
        }
        const hasChanged = await collector.handleWatchEvent(event, pathAbsolute);
        if (hasChanged && rpc) {
          rpc.broadcast.documentsUpdated([...collector.rpcItems.values()]);
        }
      });
    }
  }
});

export { module as default };
